package main

import (
	"fmt"
	"log"
	"os"

	cnf "git.pbiernat.io/egommerce/go-api-pkg/config"

	"git.pbiernat.io/egommerce/order-service/internal/app"
	"git.pbiernat.io/egommerce/order-service/internal/worker"
)

func main() {
	if cnf.ErrLoadingEnvs != nil {
		log.Fatalln("Error loading .env file.")
	}

	c := worker.NewConfig("order-worker")
	cArr := c.GetArray()

	doer := worker.New(c)
	a := app.NewApp(doer)
	a.RegisterPlugin(app.LoggerPlugin(cArr))
	a.RegisterPlugin(app.CachePlugin(cArr))
	a.RegisterPlugin(app.DatabasePlugin(cArr))
	a.RegisterPlugin(app.EventbusPlugin(cArr))

	while := make(chan struct{})
	err := a.Start(while)
	<-while

	if err != nil {
		log.Fatalf("Failed to start worker. Reason: %v\n", err)
		os.Exit(1)
	}

	fmt.Println("Gone")
	os.Exit(0)
}
